package com.jay.monitor.data.client.interceptor;

import com.jay.monitor.data.client.MonitorSendProducer;
import com.jay.monitor.data.client.api.DefaultIdSupplier;
import com.jay.monitor.data.client.api.DefaultIndexData;
import com.jay.monitor.data.client.api.MQIndexCallback;
import com.jay.monitor.data.client.utils.JaySpringApiHelper;
import com.jay.monitor.data.core.enums.MQInvokeType;
import com.jay.monitor.data.core.model.serializable.MQDataDTO;
import com.jay.monitor.data.core.utils.ObjectHelper;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageProperties;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StopWatch;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Date;
import java.util.Map;
import java.util.function.Supplier;

/**
 * 负责拦截rabbitMq相关的参数
 *
 * @author liukx
 * @date 2022/7/1 16:49
 */
@Aspect
public class JayRabbitAspect {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired(required = false)
    private Supplier<String> supplier = new DefaultIdSupplier();

    @Autowired(required = false)
    private MQIndexCallback callback = new MQIndexCallback() {
        @Override
        public DefaultIndexData mqCallback(Message message, String body) {
            return new DefaultIndexData();
        }
    };

    @Pointcut("@annotation(org.springframework.amqp.rabbit.annotation.RabbitListener)")
    public void consumerListener() {

    }

    @Around("consumerListener()")
    public Object consumerAround(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {

        MethodSignature methodSignature = (MethodSignature)proceedingJoinPoint.getSignature();
        Method method = methodSignature.getMethod();
        RabbitListener annotation = method.getAnnotation(RabbitListener.class);
        String[] queues = annotation.queues();
        String queueName = Arrays.toString(queues);

        String methodName = methodSignature.getName();
        String className = methodSignature.getDeclaringTypeName();

        String fullName = className + "#" + methodName + ":" + queueName;

        Object value = null;
        Message message = getMessage(proceedingJoinPoint.getArgs());
        try {
            StopWatch stopWatch = new StopWatch();
            stopWatch.start();
            value = proceedingJoinPoint.proceed();
            stopWatch.stop();

            invokeSuccess(message, stopWatch, value);

        } catch (Throwable e) {
            logger.error("rabbit mq error", e);
            invokeError(message, e);
            throw e;
        }
        return value;
    }

    private void invokeError(Message message, Throwable throwable) {
        if (message == null || JaySpringApiHelper.isFilterException(throwable)) {
            return;
        }
        MQDataDTO dto = builderMqDto(message);
        dto.setStatus(-1);
        dto.setRequestTime(-1L);
        dto.setErrorMessage(throwable.getMessage());
        dto.setErrorStackTrace(ObjectHelper.errorToString(throwable));
        MonitorSendProducer.sendMsg(dto);
    }

    private void invokeSuccess(Message message, StopWatch stopWatch, Object value) {
        if (message == null) {
            return;
        }
        MQDataDTO dto = builderMqDto(message);
        dto.setStatus(1);
        dto.setRequestTime(stopWatch.getTotalTimeMillis());
        MonitorSendProducer.sendMsg(dto);
    }

    private MQDataDTO builderMqDto(Message message) {
        MessageProperties messageProperties = message.getMessageProperties();
        String receivedRoutingKey = messageProperties.getReceivedRoutingKey();
        String consumerQueue = messageProperties.getConsumerQueue();
        String receivedExchange = messageProperties.getReceivedExchange();
        String body = new String(message.getBody());
        MQDataDTO dto = new MQDataDTO();
        dto.setInvokeType(MQInvokeType.consumer);
        dto.setSourceTime(messageProperties.getTimestamp());
        dto.setJsonContent(body);
        dto.setCreateDate(new Date());
        dto.setMqType("rabbitMQ");
        dto.setLogId(supplier.get());
        dto.setMsgId(messageProperties.getMessageId());
        dto.setTopic(receivedExchange);
        dto.setPartitionName(consumerQueue);
        dto.setDataId(receivedRoutingKey);
        DefaultIndexData defaultIndexData = callback.mqCallback(message, body);
        dto.setGroupName(defaultIndexData.getGroupName());
        dto.setGroupKeyName(defaultIndexData.getGroupKeyName());
        dto.setDataId(defaultIndexData.getDataId());
        Map<String, Object> extMap = defaultIndexData.getExtMap();
        dto.setExtMap(extMap);
        return dto;
    }

    private Message getMessage(Object[] args) {
        for (Object arg : args) {
            if (arg instanceof Message) {
                return (Message)arg;
            }
        }
        return null;
    }

}