package com.jay.monitor.data.server.job.impl;

import com.jay.monitor.data.server.config.props.MonitorProperties;
import com.jay.monitor.data.server.costs.ServerConstants;
import com.jay.monitor.data.server.enums.DataSplitType;
import com.jay.monitor.data.server.enums.GlobalConfigCodeEnums;
import com.jay.monitor.data.server.enums.GlobalConfigGroupEnums;
import com.jay.monitor.data.server.enums.StoreType;
import com.jay.monitor.data.server.job.ScheduleJobServiceAdaptor;
import com.jay.monitor.data.server.models.store.GlobalBaseConfig;
import com.jay.monitor.data.server.store.es.ElasticGlobalBaseConfigProcess;
import com.jay.monitor.data.server.utils.MonitorEnvHelper;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.elasticsearch.action.admin.indices.alias.get.GetAliasesRequest;
import org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest;
import org.elasticsearch.action.support.master.AcknowledgedResponse;
import org.elasticsearch.client.GetAliasesResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.cluster.metadata.AliasMetaData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 清理历史数据
 *
 * @author liukaixiong
 * @date 2022/7/26 - 13:34
 */
public class ESClearHistoryService extends ScheduleJobServiceAdaptor {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private RestHighLevelClient client;

    @Autowired
    private ElasticGlobalBaseConfigProcess configProcess;

    @Autowired
    private MonitorProperties monitorServer;

    @Override
    public boolean isDayInvoke() {
        return true;
    }

    @Override
    public void processDayJob() throws Exception {

        // 如果没有配置ES的表内容，则不走
        Map<String, DataSplitType> tableRuleMap = MonitorEnvHelper.getTableRuleMap(StoreType.ES);

        if (MapUtils.isEmpty(tableRuleMap)) {
            return;
        }

        Map<String, Set<AliasMetaData>> map = getAllIndex();

        if (map == null || map.size() == 0) {
            return;
        }

        Set<String> indices = map.keySet();

        if (!indices.contains(MonitorEnvHelper.getEnvPrefixName(ServerConstants.INDEX_GLOBAL_CONFIG_NAME))) {
            initGlobalConfigData();
        }

        int maxSaveDay = -MonitorEnvHelper.saveHistoryDay();

        // 小于该时间的都认为是过期的.
        String startDay = DateFormatUtils.format(DateUtils.addDays(new Date(), maxSaveDay), "yyMMdd");
        for (String table : tableRuleMap.keySet()) {
            String envPrefixName = MonitorEnvHelper.getEnvPrefixName(table);
            if (StringUtils.isNotEmpty(envPrefixName)) {
                // 删选出符合的
                List<String> matchTable =
                    indices.stream().filter(index -> index.startsWith(envPrefixName)).sorted(Comparator.naturalOrder())
                        .collect(Collectors.toList());
                matchTable.forEach(tableName -> {
                    String tableDay = tableName.substring(envPrefixName.length() + 1);
                    // 过滤掉过期的
                    if (isMatch(startDay, tableDay)) {
                        dropIndex(tableName);
                    }
                });
            }
        }
    }

    private void initGlobalConfigData() {
        for (GlobalConfigCodeEnums codeEnums : GlobalConfigCodeEnums.values()) {
            GlobalBaseConfig index = new GlobalBaseConfig();
            index.setGroup(GlobalConfigGroupEnums.GLOBAL_CONFIG_PROPS.name());
            index.setCode(codeEnums.name());
            Object value = codeEnums.getFunction().apply(monitorServer);
            if (value != null) {
                index.setValue(value.toString());
            }
            index.setDescribe(codeEnums.getLabel());
            index.setCreateDate(new Date());
            index.setStatus(1);
            index.setProject(ServerConstants.PROJECT_ALL_MATCH);
            configProcess.save(index);
        }
    }

    /**
     * 获取所有的index
     *
     * @return
     * @throws Exception
     */
    private Map<String, Set<AliasMetaData>> getAllIndex() throws Exception {
        GetAliasesRequest request = new GetAliasesRequest();
        GetAliasesResponse getAliasesResponse = client.indices().getAlias(request, RequestOptions.DEFAULT);
        return getAliasesResponse.getAliases();
    }

    /**
     * 删除索引
     *
     * @param tableName
     */
    private void dropIndex(String tableName) {
        DeleteIndexRequest deleteIndexRequest = new DeleteIndexRequest(tableName);
        try {
            AcknowledgedResponse delete = client.indices().delete(deleteIndexRequest, RequestOptions.DEFAULT);
            if (delete.isAcknowledged()) {
                logger.info("清理掉过期ES索引 >>> {}", tableName);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.warn("清理索引失败:{}", e.getMessage());
        }
    }

    private boolean isMatch(String startDay, String tableDay) {
        try {
            return Long.parseLong(startDay) > Long.parseLong(tableDay);
        } catch (Exception e) {
            return false;
        }
    }
}
